@@ -3,8 +3,14 @@ package org.gcube.portal.databook.server;
 import java.util.List;
 import java.util.Map;
 
+import javax.mail.internet.AddressException;
+
+import org.gcube.portal.databook.shared.Attachment;
 import org.gcube.portal.databook.shared.Comment;
 import org.gcube.portal.databook.shared.Feed;
+import org.gcube.portal.databook.shared.Invite;
+import org.gcube.portal.databook.shared.InviteOperationResult;
+import org.gcube.portal.databook.shared.InviteStatus;
 import org.gcube.portal.databook.shared.Like;
 import org.gcube.portal.databook.shared.Notification;
 import org.gcube.portal.databook.shared.NotificationChannelType;
@@ -14,6 +20,8 @@ import org.gcube.portal.databook.shared.ex.ColumnNameNotFoundException;
 import org.gcube.portal.databook.shared.ex.CommentIDNotFoundException;
 import org.gcube.portal.databook.shared.ex.FeedIDNotFoundException;
 import org.gcube.portal.databook.shared.ex.FeedTypeNotFoundException;
+import org.gcube.portal.databook.shared.ex.InviteIDNotFoundException;
+import org.gcube.portal.databook.shared.ex.InviteStatusNotFoundException;
 import org.gcube.portal.databook.shared.ex.LikeIDNotFoundException;
 import org.gcube.portal.databook.shared.ex.NotificationChannelTypeNotFoundException;
 import org.gcube.portal.databook.shared.ex.NotificationIDNotFoundException;
@@ -23,7 +31,6 @@ import org.gcube.portal.databook.shared.ex.PrivacyLevelTypeNotFoundException;
 /**
  * @author Massimiliano Assante ISTI-CNR
  * 
- * @version 1.0 Feb 2013
  * <class>DatabookStore</class> is the high level interface for querying and adding data to DatabookStore 
  */
 public interface DatabookStore {
@@ -57,6 +64,14 @@ public interface DatabookStore {
 	 * @return true if everything went fine
 	 */
 	boolean saveUserFeed(Feed feed);
+	/**
+	 * Save a Feed instance in the store having more than one attachment
+	 * Use this if you need to attach more than one file to the post
+	 *
+	 * @param attachments, a list of attachments starting from the second
+	 * @return true if everything went fine
+	 */
+	boolean saveUserFeed(Feed feed, List<Attachment> attachments);
 	/**
 	 * delete a Feed from the store
 	 * @throws FeedIDNotFoundException
@@ -64,7 +79,7 @@ public interface DatabookStore {
 	 */
 	boolean deleteFeed(String feedid) throws FeedIDNotFoundException, PrivacyLevelTypeNotFoundException, FeedTypeNotFoundException, ColumnNameNotFoundException;
 	/**
-	 *  save a Feed in the VRES TimeLine in the store
+	 *  save a post in the VRES TimeLine in the store
 	 * @param feedKey feedKey
 	 * @param vreid vre identifier
 	 * @return
@@ -72,10 +87,18 @@ public interface DatabookStore {
 	 */
 	boolean saveFeedToVRETimeline(String feedKey, String vreid) throws FeedIDNotFoundException;
 	/**
-	 * save a Feed instance in the store
+	 * save a Post instance in the store
 	 * @return true if everything went fine
 	 */
-	boolean saveAppFeed(Feed feed);
+	boolean saveAppFeed(Feed feed);	
+	/**
+	 * Save a feed instance in the store
+	 * Use this if your app needs to attach more than one file to the post
+	 *
+	 * @param attachments, a list of attachments starting from the second
+	 * @return true if everything went fine
+	 */
+	boolean saveAppFeed(Feed feed, List<Attachment> attachments);
 	/**
 	 * read a feed from a given id 
 	 * @throws PrivacyLevelTypeNotFoundException 
@@ -151,6 +174,12 @@ public interface DatabookStore {
 	 * @throws ColumnNameNotFoundException 
 	 */
 	RangeFeeds getRecentFeedsByVREAndRange(String vreid,  int from, int quantity) throws PrivacyLevelTypeNotFoundException, FeedTypeNotFoundException, ColumnNameNotFoundException, FeedIDNotFoundException;
+	/**
+	 * @param userid user identifier
+	 * @param timeInMillis time in milliseconds from which you want to start retrieve the feeds
+	 * @return the number of feeds in the range from: today to: timeInMillis
+	 */
+	List<Feed> getRecentFeedsByUserAndDate(String userid, long timeInMillis) throws IllegalArgumentException;
 	/**
 	 * save a Notification instance in the store
 	 * @return true if everything went fine
@@ -233,7 +262,7 @@ public interface DatabookStore {
 	 * @return true if there are unread messages notifications (including messages), false if they are all read
 	 */
 	boolean checkUnreadMessagesNotifications(String userid)  throws NotificationIDNotFoundException, NotificationTypeNotFoundException, ColumnNameNotFoundException;
-	
+
 	/**
 	 * return the channels a user chose for being notified for a given notification type
 	 * @param userid  user identifier
@@ -242,7 +271,7 @@ public interface DatabookStore {
 	 */
 	List<NotificationChannelType> getUserNotificationChannels(String userid, NotificationType notificationType) throws NotificationChannelTypeNotFoundException, NotificationTypeNotFoundException;
 	/**
-	 * set the notification preferences map (enableor disable the channels to be used for notifying the user)
+	 * set the notification preferences map (enable or disable the channels to be used for notifying the user)
 	 * @param userid  user identifier
 	 * @param notificationType the type of the notification
 	 * @param enabledChannels a map of the channels to which reach the user per notification, empty array or null values to set the key notification type off
@@ -258,6 +287,12 @@ public interface DatabookStore {
 	 */
 	Map<NotificationType, NotificationChannelType[]> getUserNotificationPreferences(String userid) throws NotificationTypeNotFoundException, NotificationChannelTypeNotFoundException;
 	
+	/**
+	 * @param commentId comment unique identifier
+	 * @return the comment belonging to the commentId
+	 * @throws CommentIDNotFoundException
+	 */
+	Comment readCommentById(String commentId) throws CommentIDNotFoundException;
 	/**
 	 * add a comment to a feed
 	 * @param comment the Comment instance to add
@@ -278,7 +313,7 @@ public interface DatabookStore {
 	/**
 	 * delete a comment 
 	 * @param commentid the comment identifier to delete
-	 * @parma feedid the feedid to shich the comment is associated
+	 * @parma feedid the feedid to which the comment is associated
 	 * @return true if success, false otherwise
 	 */
 	boolean deleteComment(String commentid, String feedid) throws PrivacyLevelTypeNotFoundException, FeedTypeNotFoundException, ColumnNameNotFoundException, CommentIDNotFoundException, FeedIDNotFoundException;
@@ -292,7 +327,7 @@ public interface DatabookStore {
 	 * unlike a feed 
 	 * @param userid user identifier
 	 * @param likeid the like identifier to delete
-	 * @param feedid the feedid to shich the comment is associated
+	 * @param feedid the feedid to which the comment is associated
 	 * @return true if success, false otherwise
 	 */
 	boolean unlike(String userid, String likeid, String feedid) throws PrivacyLevelTypeNotFoundException, FeedTypeNotFoundException, ColumnNameNotFoundException, LikeIDNotFoundException, FeedIDNotFoundException;
@@ -317,7 +352,81 @@ public interface DatabookStore {
 	 * return all the likes belonging to the feedid
 	 */
 	List<Like> getAllLikesByFeed(String feedid);
-
+	/**
+	 * 
+	 * @param hashtags the hashtag including the '#'
+	 * @param feedid the feedid to which the hashtag is associated
+	 * @param vreid VRE identifier
+	 * @return true if success, false otherwise
+	 * @throws FeedIDNotFoundException
+	 */
+	boolean saveHashTags(String feedid, String vreid, List<String> hashtags) throws FeedIDNotFoundException;
+	/**
+	 * 
+	 * @param hashtags the hashtag including the '#'
+	 * @param feedid the feedid to which the hashtag is associated
+	 * @param vreid VRE identifier
+	 * @return true if success, false otherwise
+	 * @throws FeedIDNotFoundException
+	 */
+	boolean deleteHashTags(String feedid, String vreid, List<String> hashtags) throws FeedIDNotFoundException;
+	/**
+	 * get a map of vre hashtags where the key is the hashtag and the value is the occurrence of the hashtag in the VRE
+	 * @param vreid vre identifier (scope)
+	 * @return a HashMap<String, Integer> of vre Hashtags associated with their occurrence
+	 */
+	Map<String, Integer> getVREHashtagsWithOccurrence(String vreid);
+	/**
+	 * 
+	 * @param vreid VRE identifier
+	 * @param hashtag the hashtag to look for including the '#', it is case sensitive
+	 * @throws ColumnNameNotFoundException .
+	 * @throws FeedIDNotFoundException .
+	 * @throws FeedTypeNotFoundException . 
+	 * @throws PrivacyLevelTypeNotFoundException 
+	 * @throws FeedIDNotFoundException .
+	 * @return all the feeds having the hashtag passed as parameter 
+	 */
+	List<Feed> getVREFeedsByHashtag(String vreid, String hashtag) throws PrivacyLevelTypeNotFoundException, FeedTypeNotFoundException, FeedIDNotFoundException, ColumnNameNotFoundException;
+	/**
+	 * Save the invite for a given email into a given vre
+	 * @param invite the invite object instanc to save
+	 * @return {@link InviteOperationResult}  SUCCESS, FAILED or ALREADY_INVITED (if an invite is sent to en existing email in the same environment more than once)
+	 */
+	InviteOperationResult saveInvite(Invite invite) throws AddressException;
+	/**
+	 * 
+	 * @param vreid the environment where you want to check the invite
+	 * @param email the email of the invite to check in the environmnet
+	 * @return the InviteId if present, null otherwise
+	 */
+	String isExistingInvite(String vreid, String email);
+	/**
+	 * read an invite from a given id 
+	 * @throws InviteIDNotFoundException 
+	 * @throws InviteStatusNotFoundException 
+	 */	
+	Invite readInvite(String inviteid) throws InviteIDNotFoundException, InviteStatusNotFoundException;
+	/**
+	 * set the status of an invite, see {@link InviteStatus}
+	 * @throws InviteIDNotFoundException 
+	 */	
+	boolean setInviteStatus(String vreid, String email, InviteStatus status) throws InviteIDNotFoundException, InviteStatusNotFoundException;
+	/**
+	 * Use to get the list of invites per VRE
+	 * @param vreid the vre id
+	 * @param status optional, if you want to restict on the status, e.g. all pending invites 
+	 * @return return the list of invites 
+	 * @throws InviteIDNotFoundException 
+	 * @throws InviteStatusNotFoundException 
+	 */
+	List<Invite> getInvitedEmailsByVRE(String vreid, InviteStatus... status) throws InviteIDNotFoundException, InviteStatusNotFoundException;
+	/**
+	 * 
+	 * @param feedId
+	 * @return the list of attachments of the feed feedId, starting from the second one (first attachment is included in Feed instance already)
+	 */
+	List<Attachment> getAttachmentsByFeedId(String feedId) throws FeedIDNotFoundException;
 	/**
 	 * close the connection to the underlying database
 	 */
